#################################################################################
# Created       : Martin Foster
# Modified      : 08/28/2001
#################################################################################
#
# Post - Module part of Webchat designed to display posts based on information
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

package Ethereal::Post;						# Package name

use strict;							# Strict variable enforcement


#################################################################################
# Data Members
#################################################################################

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
        my $class = shift;
        my $self  = { };

        # Bless $self into designated class
        bless $self, $class;

        # Return self
        return $self;
}


#####################
# Get Posts
#
# Using basic information about the user will return a Statement handle (more 
# lightweight then an array) that will be used in the display the information
# gathered.

sub GetPosts
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $database  = shift;					# Database handle
	my $statement = shift;					# Statement handle (reference)
	my $param     = shift;					# Parameter list
	my $realm     = shift;					# Reference to realm hash

	my $data      = $database->{'HANDLE'};			# Database connection handle

	my $formatting;						# Formatting used	
	my $delay;						# Delay to retreive posts

	my $ignored;						# Ignored person
		

	#####################
	# Program area

	# Determine action 
	if ((!defined($param->{'NUM'})) || (defined($param->{'SCROLLBACK'})))
	{ 
		# Time of delay
		$delay = (time - $realm->{'SetTimeout'}); 

		# Cleanup
		$param->Cleanup('SCROLLBACK');
	}	


	# Format handling
	# Retreive formatting used
	($formatting) = $database->DataGetFormatting($param->{'USER'});

	# Determin field to use
	# Full formatting
	if ($formatting eq 'complete')	
	{ $formatting = 'PostFullFormat'; }

	# Spartan formatting
	elsif ($formatting eq 'spartan')
	{ $formatting = 'PostPartialFormat'; }

	# Full less images
	elsif ($formatting eq 'imageless')
	{ $formatting = 'PostImagelessFormat'; }


	# List of ignored
	$ignored = $self->IgnoreHandler($database, $param);

	# Number based search
	unless (defined($delay))
	{
		# Initialize and execute
		${$statement} = $data->prepare("SELECT PostIDNumber, $formatting FROM Post 
					WHERE PostIDNumber > ? AND RealmName=? AND (PostTo='all' OR PostTo=?) 
					$ignored ORDER BY PostIDNumber");
		${$statement}->execute($param->{'NUM'}, $param->{'ROOM'}, $param->{'USER'});
	}
	# Time based search
	else
	{
		# Initialize and execute
		${$statement} = $data->prepare("SELECT PostIDNumber, $formatting FROM Post 
					WHERE PostTimeStamp > ? AND RealmName=? AND (PostTo='all' OR PostTo=?) 
					$ignored ORDER BY PostIDNumber");
		${$statement}->execute($delay, $param->{'ROOM'}, $param->{'USER'});
	}
}


#####################
# Handle Posts
#
# Will display all retreived posts to screen, a fairly straightforward 
# operation, will also give a value to NUM once it knows the last post
# received

sub HandlePosts
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter list

	my $statement;						# Query statement

	my $post;						# Post itself
	my $postnum;						# Post number

	my %realm;						# Realm hash


	#####################
	# Program area

	# Link with hash
	$database->GetHashRealm(\%realm);

	# Retreive posts
	$self->GetPosts($database, \$statement, $param, \%realm);	

	# Run though posts and print
	while ( ($_, $post) = $statement->fetchrow() )
	{ 
		print "$realm{'TagSeperator'}";			# Seperator line
		print $post;					# Display post 
		$postnum = $_;					# Give value to postnum
	}

	# Finish query
	$statement->finish();
	
	# Change value of NUM or define
	if (defined($postnum))
	{ $param->{'NUM'} = ($postnum - 1); }
}


#####################
# Handle Stream
#
# Will display all retreived posts to screen, a fairly straightforward 
# operation.  This unlike HandlePosts will not stop until the stream
# is ended

sub HandleStream
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI Handle
	my $param    = shift;					# Parameter list

	my $statement;						# Query statement

	my $post;						# Post itself
	my $postnum;						# Post number
	my $postold = 0;					# Old post number
	my $tmp;						# Temporary scalar holder

	my $size    = 0;					# Size of posts
	my $active  = 0;					# Activity counter
	my $elapse  = 0;					# Time elapsed		

	my $seperator;						# Seperator line

	my %realm;						# Chat hash


	#####################
	# Program area

	# Assign seperator
	$database->GetHashRealm(\%realm);
	$seperator = $realm{'TagSeperator'};

	# As long as conditions are not met
	while (($size  < $realm{'SetStreamSize'}) 
	   && ($active < $realm{'SetActive'})
	   && ($elapse < $realm{'SetElapse'}))
	{
		# Retreive posts
		$self->GetPosts($database, \$statement, $param, \%realm);	

		# Run though posts and print
		while ( ($tmp, $post) = $statement->fetchrow() )
		{ 
			# Display
			print "$seperator";			# Seperator line
			print $post;				# Display post 
	
			# Assignment
			$postnum = $tmp;			# Give value to postnum
			$size += length($post);
		}

		# Finish query
		$statement->finish();
	
		# Change value of NUM or define
		if (defined($postnum))
		{ 
			# Increment 
			$active = ($postold != $postnum ) ? 0 : ($active + 1); 

			# Assign PostOld and Parameter
			$postold 	= $postnum;
			$param->{'NUM'} = $postnum;
		}

		# Print comment
		print $cgi->comment("$postnum"), "\n";

		# Final size calculations
		$size += (length($postnum) + 8);
		
		# Increment Elapsed time
		$elapse++;

		# Sleep before next pass
		sleep(5);
	}

	# To retrieve last made post
	$param->{'NUM'}--;


	# Activation button
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'MAIN', 'True'), "\n";

	# Table information
	print "<BR>\n<CENTER>", $cgi->submit($realm{'SetPost'}), "</CENTER>\n";

	# End of form
	print $cgi->end_form(), "\n";
}


#####################
# Ignore Handler
#
# Will create a string to append to the end of the SQL statement, as
# to ignore the specified list of people.

sub IgnoreHandler
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter list

	my $string;						# Ignored string
	my $ignored;						# Ignored person
	my @ignored;						# Ignored group

	#####################
	# Program area

	# Retreive list of the damned
	$database->GetListPuppetIgnored(\@ignored, $param->{'USER'});

	# As long as there are entries
	while ($ignored = shift @ignored)
	{
		# Add double quotes if necessary
		$ignored =~ s/'/''/gs;

		# If the string is defined
		if (defined($string))
		{ $string = "$string AND PuppetName <> '$ignored'"; }

		# Initialize for next pass
		else
		{ $string = " AND PuppetName <> '$ignored'"; }		
	}

	# Checks before returning
	if (defined($string))
	{ return $string;  }					# Return string

	else							# Empty string
	{ return ' '; } 
}

1;								# Return true
