#!/bin/perl -w
#################################################################################
# Created       : Martin Foster
# Modified      : 08/30/2001
#################################################################################
#
# Webchat Command line - Command line controls  part of Webchat
# Copyright (C) 2000  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5Th Street North-East
#                 Calgary, Alberta
#                 T2R 3W9
#
#################################################################################
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Date;						# Date handler
use Ethereal::Home;						# Home page handler
use Ethereal::Weather;						# Weather forcasts

#################################################################################
# Data Members
#################################################################################
my $database;							# Database handle
my $func = shift @ARGV;						# Function command
my $type = shift @ARGV;						# Type to expire

#################################################################################
# Program Area
#################################################################################



	# Only continue if parameters are sent
	if (defined($func))
	{
		# Connect to Database
		$database = new Ethereal::Database();
		$database->Connect();

		# List audit entries
		if ($func eq '-audit')
		{ ListAudit($database, $type); }

		# List crypt
		if ($func eq '-crypt')
		{ ListCrypt($database); }

		# Listing of Emails
		if ($func eq '-list')
		{ ListEmail($database); }

		# Realm stats
		if ($func eq '-realm')
		{ ListRealms($database); }

		# Listing useage summary
		if ($func eq '-summary')
		{ ListSummary($database); }

		# Forcast update
		if ($func eq '-forcast')
		{ ForcastUpdate($database); }
	
		# Expiery control
		if ($func eq '-expire')
		{
			# Specific expire
			if (defined($type))
			{
				# Banings
				if ($type eq 'ban')
				{ ExpireBans($database); }

				# Remove old internal mail
				if ($type eq 'mail')
				{ ExpireMail($database); }

				# Posts
				elsif ($type eq 'post')
				{ ExpirePosts($database); }

				# Puppeteers
				elsif ($type eq 'puppeteer')
				{ ExpirePuppeteer($database); }

				# Puppets
				elsif ($type eq 'puppet')
				{ ExpirePuppet($database); }

				# Private realms
				elsif ($type eq 'realm')
				{ ExpireRealm($database); }

				# Registrations
				elsif ($type eq 'registration')
				{ ExpireRegistrations($database); }
			}

			# General expire
			else
			{
				# Run though all methods
				ExpireBans($database);
				ExpireMail($database);
				ExpirePosts($database);
				ExpirePuppeteer($database);
				ExpirePuppet($database);
				ExpireRealm($database);
				ExpireRegistrations($database);
			}
		}
	}

	# Display help
	unless (defined($func))
	{
		print "Webchat Command-Line Interface:\n";
		print "Usage: webchatcmd.plx {-audit|-crypt|-expire|-list|-summary} [expire]\n\n";
		print "-audit [days]\n";
		print "\tDisplay contents of audit table\n\n";
		print "-crypt\n";
		print "\tProduces output for Apache password files\n\n";
		print "-list\n";
		print "\tDisplay a list of all E-Mail addresses of members\n\n";
		print "-forcast\n";
		print "\tUpdates the forcast information for all realms making use of it\n\n";
		print "-realm\n";
		print "\tSummary for a realms specifically\n\n";
		print "-summary\n";
		print "\tSummary of room useage\n\n";
		print "-expire\n";
		print "\tWill remove expired entires.  When called alone all are removed\n";
		print "\tor if specified only a specific task will take place as listed\n";
		print "\tbelow:\n";
		print "\t\tban          - Remove old bannings\n";
		print "\t\tmail         - Remove old mail\n";
		print "\t\tpost         - Remove old posts\n";
		print "\t\tpuppet       - Remove inactive puppets\n";
		print "\t\tpuppeteer    - Remove inactive puppeteers\n";
		print "\t\trealm        - Remove inactive private realms\n";
		print "\t\tregistration - Remove old incomplete registrations\n\n";
	}

#################################################################################
# Sub-Routines
#################################################################################

####################
# List Audit
#
# Produces an output, which can be used for a text file containing all of the 
# audit entires in the database itself.   This will of course work from lastest
# to earliest entries.

sub ListAudit
{
	####################
	# Data members
	my $database = shift;					# Database handle
	my $days     = shift;					# Elapsed days

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $statement;						# Database statement

	my $date     = new Ethereal::Date();			# Date handler

	my $audit;						# Action completed


	####################
	# Program area

	# Retrieve search date
	$days = (defined($days)) ? (time - ($days * 86400)) : 0;
	$days = $date->GetDate($days);

	# Prepare and execute
	$statement = $data->prepare("SELECT 
			  AuditTimestamp,
			  PuppeteerLogin,
			  PuppeteerHostname,
			  AuditAction
			FROM  Audit
			WHERE AuditTimeStamp > ?"); 
	$statement->execute($days);

	# Audit trail
	while ($audit = $statement->fetchrow_hashref())
	{
		print "$audit->{AuditTimestamp} ($audit->{PuppeteerLogin} \@ $audit->{PuppeteerHostname}) - $audit->{AuditAction}\n";
	}

	# Finsih call
	$statement->finish();
}

####################
# List Crypt
#
# Lists the encrypted usernames and passwords for supervisors and the
# administrators to allow for building password files for Apache.

sub ListCrypt
{
	####################
	# Data Members
	my $database = shift;					# Database handle

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $statement;						# Database statement

	my $crypt;						# Action completed

	
	####################
	# Program area

	# Prepare and execute
	$statement = $data->prepare("SELECT 
			  PuppeteerLogin,
			  PuppeteerPassword
			FROM  Puppeteer
			WHERE PuppeteerAdminPrivs='yes'");
	$statement->execute();

	# Crypted Password
	while ($crypt = $statement->fetchrow_hashref())
	{
		print "$crypt->{PuppeteerLogin}:$crypt->{PuppeteerPassword}\n";
	}

	# Finsih call
	$statement->finish();
}

####################
# List EMail
#
# Is used to simply list of all E-Mail addresses of those who are members of the
# chat server.  This is useful to embed within Majordomo or other such mailing
# list system.

sub ListEmail							# Database
{
	####################
	# Data members
	my $database = shift;					# Database handle

	my $puppeteers;						# Single puppeteer EMail
	my @puppeteers;						# List of all EMails


	####################
	# Program area

	# Retreive puppeteers E_Mail addresses
	$database->GetListEMails(\@puppeteers);

	# Print each E-Mail address
	foreach $puppeteers (@puppeteers)
	{ print "$puppeteers\n"; }
}


####################
# Realm specific summary
#
# Realm specific summaries that will be mailed specifically to the hosts
# that are entrusted with the realms themselves.  This keeps them from
# comparing with other realms in service.

sub ListRealms
{
	####################
	# Data members
	my $database = shift;					# Database handle

	my $date     = new Ethereal::Date();			# Date handle

	my $elapsed;						# Elapsed time
	my $end;						# End of search span
	my $start;						# Begin of search span

	my $lhits;						# Local hits in a realm
	my $lposts;						# Local posts in a realm
	my $ghits;						# General hits in a realm
	my $gposts;						# General posts in a realm

	my $count;						# Counter
	my $host;						# Individual host
	my $realm;						# Individual realm
	my $who;						# Individual person

	my @hosts;						# List of hosts
	my @realms;						# List of realms
	my @whos;						# Who was on

	my %system;						# System hash


	####################
	# Program area

	# Data retreival
	# Link with hash
	$database->GetHashSystem(\%system);

	# Retreive realms
	$database->GetList(\@realms, "SELECT RealmName 
		FROM RealmSupervisor 
		GROUP BY RealmName");


	# Run through list
	foreach $realm (sort(@realms))
	{
		# Calculate elapsed time
		$elapsed   = time - 86400;


		# Time based statistics
		$start = $date->GetDateShort($elapsed);
		$end   = $date->GetDateShort(time);


		# Data Retreival
		# Single Values
		# Posts
		($gposts) = $database->DataGet("SELECT COUNT(*) 
			FROM Post 
			WHERE RealmName=?
			  AND PostTimestamp > ?
			  AND PostTo <> 'system'", $realm, $elapsed);
		$gposts = (defined($gposts)) ? $gposts : 0;

		# Hits
		($ghits)  = $database->DataGet("SELECT COUNT(*)
			FROM Post
			WHERE RealmName=?
			  AND PostTimestamp > ?", $realm, $elapsed);
		$ghits  = (defined($ghits)) ? $ghits : 0;


		# List values
		# Retreive hosts
		$database->GetList(\@hosts, "SELECT p.PuppeteerEmail 
			FROM  Puppeteer p, 
			      RealmSupervisor r 
			WHERE p.PuppeteerLogin=r.PuppeteerLogin 
                          AND r.RealmName=?", $realm);


		# Initiate Email composition
		# Define and open sendmail
		open(SENDMAIL, "|$system{'LocSendmail'} -oi -t") or die("Cannot fork sendmail: $!");

		# Send out header
		# From field
		print SENDMAIL "From: $system{'SetInfoContactName'} <$system{'SetInfoContactAddress'}>\n";

		# Send to each hosts
		while ($host = shift @hosts)
		{ print SENDMAIL "To: <$host>\n"; }

		# Subject line
		print SENDMAIL "Subject: Statistics for $realm from $start to $end\n\n";
		

		# Main body
		# Title line
		print SENDMAIL "Statistics for $realm from $start to $end\n\n";

		# Realm statistics
		# Hourly stats
		print SENDMAIL "Realm Statistics\n-------------------------\n\n";
		print SENDMAIL "Time\tPosts\tHits\n-----\t------\t------\n";
	
		# Cycle through
		for ($count=1; $count < 25; $count++)
		{
			# Search ranges
			$end   = ($elapsed + ($count * 3600));
			$start = ($end - 3600);

			# Queries
			# Posts
			($lposts) = $database->DataGet("SELECT COUNT(*) 
				FROM Post 
				WHERE RealmName=?
				  AND PostTimestamp > ?
				  AND PostTimestamp < ?
				  AND PostTo <> 'system'", $realm, $start, $end);
			$lposts = (defined($lposts)) ? $lposts : 0;

			# Hits
			($lhits)  = $database->DataGet("SELECT COUNT(*)
				FROM Post
				WHERE RealmName=?
				  AND PostTimestamp > ?
				  AND PostTimestamp < ?", $realm, $start, $end);
			$lhits  = (defined($lhits)) ? $lhits : 0;


			# Reuse start
			$start = $date->GetTime($start);

			# Display of information
			print SENDMAIL "$start\t$lposts\t$lhits\n";

		}

		# General Information
		print SENDMAIL "General\t$gposts\t$ghits\n\n\n";


		# Who was on
		print SENDMAIL "Who was on?\n-------------------------\n\n";

		# Cycle through
		for ($count=1; $count < 25; $count++)
		{
			# Search ranges
			$end   = ($elapsed + ($count * 3600));
			$start = ($end - 3600);


			# Puppets and puppeteers
			$database->GetList(\@whos, "SELECT CONCAT(Ch.PuppetName, ' (', Pu.PuppeteerLogin, ')') AS PuppetName 
				FROM  Post Po, 
				      Puppeteer Pu, 
				      Puppet Ch 
				WHERE Po.PuppetName = Ch.PuppetName 
				  AND Pu.PuppeteerLogin = Ch.PuppeteerLogin 
				  AND Po.RealmName = ? 
				  AND Po.PostTimestamp > ? 
				  AND Po.PostTimestamp < ? 
				GROUP BY Po.PuppetName, Po.RealmName 
				ORDER BY Po.RealmName, Po.PuppetName", $realm, $start, $end);

			# Reuse start
			$start = $date->GetTime($start);

			print SENDMAIL "$start\n-------------------------\n\n";


			# Cycle and display
			while ($who = shift @whos)
			{
				print SENDMAIL "\t$who\n";
			}
		}
		# Close sendmail
		close(SENDMAIL);
	}
}


####################
# List Summary
#
# Somewhat inneficient function that will display a summary all the useage
# in all the rooms.   Public or otherwise.
#

sub ListSummary							# Database
{
	####################
	# Data members
	my $database = shift;					# Database handle

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $statement;						# Database statement

	my $number;						# Number
	my $name;						# Name of room

	my $users;						# Amount of users
	my $puppets;						# Amount of puppets
	my $realm;						# Realm name

	my $day;						# Users in day
	my $week;						# Users in week
	my $month;						# Users in month
	my $expire;						# About to expire
	my $elapsed;						# Calculation of elapsed time

	my %hits;						# Total hits
	my %posts;						# Total posts


	####################
	# Program area

	# Prepare and execute
	# Total posts
	$statement = $data->prepare("SELECT COUNT(*), 
				  RealmName 
				FROM  Post 
				GROUP BY RealmName
				ORDER BY RealmName");
	$statement->execute();

	while (($number, $name) = $statement->fetchrow())
	{ $hits{$name} = $number };

	# Finish
	$statement->finish();


	# Accurate posting
	$statement = $data->prepare("SELECT COUNT(*), 
				  RealmName 
				FROM  Post 
				WHERE PostTo <> 'system' 
				GROUP BY RealmName
				ORDER BY RealmName");
	$statement->execute();

	while (($number, $name) = $statement->fetchrow())
	{ $posts{$name} = $number };

	# Finish
	$statement->finish();


	# Room useage
	# Initial display
	print "Posts   Hits    Realm Name\n";
	print "------- ------- ----------------------------------\n";

	# Retreive and display
	foreach $name (sort(keys(%hits)))
	{
		# Ensure integrity
		$posts{$name} = (defined($posts{$name})) ? $posts{$name} : 0;
 
		# Display to screen
		print "$posts{$name}\t$hits{$name}\t$name\n"; 
	}


	# Chat Preferences
	# Initial Display
	print "\n\nHits    Chat Type\n";
	print "------- ----------------------------------\n";

	# Prepare and execute
	$statement = $data->prepare("SELECT COUNT(*), 
		  PuppeteerChatPref 
		FROM Puppeteer 
		GROUP BY PuppeteerChatPref");
	$statement->execute();

	# Retreive and display
	while (($number, $name) = $statement->fetchrow())
	{ print "$number\t$name\n"; }

	# Finish
	$statement->finish();


	# Puppet Useage
	# Initial display
	print "\n\nFreqency Count   Percent\n";
	print "-------- ------- -------\n";

	# Totals
	($users)   =  $database->DataGet('SELECT COUNT(*) FROM Puppeteer');
	($puppets) =  $database->DataGet('SELECT COUNT(*) FROM Puppet');

	# Day use
	$elapsed   = time - 86400; 
	($day)     = $database->DataGetFrenquency($elapsed);
	printf "Day      %7i   %5.2f\n", $day, (($day/$users)*100);	

	# Weekly use
	$elapsed   = time - (86400*7);
	($week)    = $database->DataGetFrenquency($elapsed);
	printf "Week     %7i   %5.2f\n", $week, (($week/$users)*100);	

	# Monthly use
	$elapsed   = time - (86400*30);
	($month)   = $database->DataGetFrenquency($elapsed);
	printf "Month    %7i   %5.2f\n", $month, (($month/$users)*100);	

	# About to expire
	$elapsed   = time - (86400*89);
	($expire)  = $database->DataGet("SELECT COUNT(*) 
		FROM Puppeteer 
		WHERE PuppeteerTimestamp < $elapsed");
	printf "Expire   %7i   %5.2f\n", $expire, (($expire/$users)*100);	

	print "\n\nTotals\n-------\nPuppeteers: $users\nPuppets: $puppets\n";
}


####################
# Expire Bans
# 
# Will expire all bans older then a certain time.

sub ExpireBans
{
	####################
	# Data members
	my $database = shift;					# Database handle
	my $data     = $database->{'HANDLE'};			# Database connection handle

	my $time;						# Stale time

	my %system;						# System hash


	####################
	# Program area

	# Remove old posts
	$data->do("DELETE FROM PuppeteerRestriction
		WHERE RestrictionTimestamp < ?", {}, time);
}

####################
# Expire mail
# 
# Will expire all mail older then a certain time.

sub ExpireMail
{
	####################
	# Data members
	my $database = shift;					# Database handle
	my $data     = $database->{'HANDLE'};			# Database connection handle

	my $time;						# Stale time

	my %system;						# System hash


	####################
	# Program area

	# Calculate time
	$database->GetHashSystem(\%system);
	$time = (time - ($system{'SetTimeoutMail'} * $system{'SetTimeDay'})); 

	# Remove old posts
	$data->do("DELETE FROM Mail 
		WHERE MailTimestamp < ?", {}, $time);
}


####################
# Expire Posts
# 
# Will expire all posts which are older then twenty minutes.

sub ExpirePosts
{
	####################
	# Data members
	my $database = shift;					# Database handle
	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $statement;						# Database statement

	my $time;						# Stale time

	####################
	# Program area

	# Calculate time
	$time = (time - 1200); 

	# Remove old posts
	$data->do("DELETE FROM Post 
		WHERE PostTimestamp < ?", {}, $time);
}


####################
# Expire Puppets
# 
# Expires inactive puppets this will occur at different intervals then the 
# expiration of the puppeteer

sub ExpirePuppet
{
	####################
	# Data members
	my $database = shift;					# Database handle
	my $data     = $database->{'HANDLE'};			# Database connection handle

	my $time;						# Stale time

	my %system;						# System hash


	####################
	# Program area

	# Calculate time for which older are destroyed
	$database->GetHashSystem(\%system);
	$time = (time - ($system{'SetTimeoutPuppet'} * $system{'SetTimeDay'}));

	# Remove old posts
	$data->do("DELETE FROM Puppet 
		WHERE PuppetActive IS NOT NULL
		  AND PuppetActive < ?", {},  $time);
}


####################
# Expire Puppeteer
#
# Will remove all puppeteers that have lapsed on their membership times.  Which
# is determine in the system DBM.

sub ExpirePuppeteer
{
	####################
	# Data members
	my $database = shift;					# Database handle
	my $data     = $database->{'HANDLE'};			# Database connection handle

	my $page     = new Ethereal::Home();			# Home page handler

	my $home;						# Home page
	my $puppeteer;						# Individual puppeteer
	my $puppet;						# Individual puppet
	my $time;						# Time in seconds
	
	my @puppeteers;						# List of puppeteers
	my @puppets;						# List of puppets

	my %home;						# Home page hash
	my %system;						# System hash


	####################
	# Program area
	
	# Link with hash
	$database->GetHashHome(\%home);
	$database->GetHashSystem(\%system);

	# Calculate time for which older are destroyed
	$time = (time - ($system{'SetTimeoutPuppeteer'} * $system{'SetTimeDay'})); 

	# Rerieve list of puppeteers
	$database->GetListExpire(\@puppeteers, $time);

	# Run though list of puppeteers	
	foreach $puppeteer (@puppeteers)
	{
		# Home page handling
		# Retreive home
		($home) = $database->DataGetPuppeteerHome($puppeteer);

		# Remove folder
		if (defined($home))
	 	{
			# Attempt to remove, else error 
			unless ($page->DirClear("$home{SetRoot}/$home")) 
			{ print "\nERROR: Home directory $home{SetRoot}/$home not cleared ($!)\n";}
		}


		# Database handling
		# Retreive list of puppets 
		$database->GetListPuppetYours(\@puppets, $puppeteer);

		# Run though list of puppets
		while ($puppet = shift @puppets)
		{
			# Remove instance of puppets
			$data->do("DELETE FROM PuppetInvitation WHERE PuppetName=?", {}, $puppet);
			$data->do("DELETE FROM PuppetIgnore     WHERE PuppetName=?", {}, $puppet);
			$data->do("DELETE FROM Puppet           WHERE PuppetName=?", {}, $puppet);
		}

		# Remove all instances of the puppeteers (except from Audit)
		$data->do("DELETE FROM Mail                  WHERE PuppeteerLogin=?", {}, $puppeteer);
		$data->do("DELETE FROM RealmSupervisor       WHERE PuppeteerLogin=?", {}, $puppeteer);
		$data->do("DELETE FROM PuppeteerRestriction  WHERE PuppeteerLogin=?", {}, $puppeteer);
		$data->do("DELETE FROM PuppeteerRegistration WHERE PuppeteerLogin=?", {}, $puppeteer);
		$data->do("DELETE FROM PuppetIgnore          WHERE PuppeteerLogin=?", {}, $puppeteer);
		$data->do("DELETE FROM Puppeteer             WHERE PuppeteerLogin=?", {}, $puppeteer);
	}
}


####################
# Expire Realm
#
# Will remove all private realms after elapsed time.

sub ExpireRealm
{
	####################
	# Data members
	my $database = shift;					# Database handle

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $statement;						# Database statement

	my $time;						# Time in seconds
	my $realm;						# Individual puppeteer
	
	my %system;						# System hash


	####################
	# Program area

	# Calculate time for which older are destroyed
	$database->GetHashSystem(\%system);
	$time = (time - ($system{'SetTimeoutRealm'} * $system{'SetTimeDay'})); 
	
	# Prepare and execute
	$statement = $data->prepare("SELECT RealmName 
		FROM  Realm 
		WHERE RealmPublic='no' 
		  AND RealmTimestamp < ?");
	$statement->execute($time);

	# Run though list
	while ($realm = $statement->fetchrow())
	{
		# Remove affected entries
		$data->do("DELETE FROM Post                 WHERE RealmName=?", {}, $realm);
		$data->do("DELETE FROM Restriction          WHERE RealmName=?", {}, $realm);
		$data->do("DELETE FROM RealmSupervisor      WHERE RealmName=?", {}, $realm);
		$data->do("DELETE FROM PuppetInvitation     WHERE RealmName=?", {}, $realm);
		$data->do("DELETE FROM PuppeteerRestriction WHERE RealmName=?", {}, $realm);		
		$data->do("DELETE FROM Realm                WHERE RealmName=?", {}, $realm);
	}

	# End query
	$statement->finish();
}


####################
# Expire Registrations
# 
# Will expire all registration older then a certain taime who have not completed
# their registration.

sub ExpireRegistrations
{
	####################
	# Data members
	my $database = shift;					# Database handle

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $statement;						# Database statement

	my $time;						# Stale time
	my $puppeteer;						# Individual puppeteer

	my %system;						# System hash


	####################
	# Program area

	# Calculate time
	$database->GetHashSystem(\%system);
	$time = (time - ($system{'SetTimeoutRegister'} * $system{'SetTimeDay'})); 


	# Initialize and execute
	$statement = $data->prepare("SELECT PuppeteerLogin
		FROM  PuppeteerRegistration 
		WHERE PuppeteerTimeStamp < ?");		
	$statement->execute($time);

	# Cycle though list
	while ($puppeteer = $statement->fetchrow())
	{
		# Remove unregistered puppeteers
		$data->do("DELETE FROM Puppet           	 WHERE PuppeteerLogin=?", {}, $puppeteer);
		$data->do("DELETE FROM Puppeteer             WHERE PuppeteerLogin=?", {}, $puppeteer);
		$data->do("DELETE FROM PuppeteerRegistration WHERE PuppeteerLogin=?", {}, $puppeteer);
	}

	# End query
	$statement->finish();
}

####################
# Update Forcast
#
# Updates the forcasts for all of the realms that are making use of this system
# allowing the hosts to get fresh weather information without a long wait.

sub ForcastUpdate
{
	####################
	# Data Members
	my $database = shift;					# Database handle
	my $data     = $database->{'HANDLE'};			# Database connection handle

	my $forc;						# Handle to weather

	my $station;						# Individual station
	my @stations;						# List of stations
	my %stations;						# Stations and forcasts
	

	####################
	# Program Area

	# Retreive list
	$database->GetList(\@stations, "SELECT RealmICAO
			FROM  Realm
			WHERE ISNULL(RealmICAO) <> 1
			GROUP BY RealmICAO");


	# Initialize handle
	$forc = new Ethereal::Weather();

	# Retreive forcasts
	%stations = $forc->Get(@stations);


	# Cycle and update database
	foreach $station (keys(%stations))
	{
		# Update with new information
		$data->do("UPDATE Realm
			SET   RealmMETAR=?
			WHERE RealmICAO=?", {},

		 $stations{$station},
		 $station);
	}
}
