#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 08/30/2001
#################################################################################
#
# Configure Puppeteer - Script part of Webchat designed to configure Puppeteers
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use Digest::MD5 qw(md5_hex);                                    # MD5 digest cryptographic algorithm
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Login;						# Login functionality
use Ethereal::Param;						# Parameter control
use Ethereal::Table;						# Table control


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter hash

my %config;							# Configuration tag

#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;	
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);

	# Fetch hash
	$database->GetHashConfiguration(\%config);

	# Document header
	print $cgi->header();
	$database->DocumentGetHeader();

	# Login handle
	$login    = new Ethereal::Login($database, $cgi, $param);

	# Authenthication
	if ($login->GetVerificationNormal())
	{
		# Print title
		print "$config{'TagTitle'}\n<BR>\n";

		# Puppet activity handling
		if (defined($param->{'PUPPET'}))
		{ ActivityHandler($database, $cgi, $param, \%config); }

		# Initial screen	
		unless (defined($param->{'OLDPASS'}) || defined($param->{'EMAIL'}))
		{ InitialDisplay($database, $cgi, $param, \%config); }

		# Information handling
		else
		{ InformationHandler($database, $cgi, $param, \%config); }
	}

	# Document footer
	$database->DocumentGetFooter();


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Activity Handler
#
# Will handle the activation de-activation of puppets as designated by the user
# this is to reduce the long list of puppets for selection when some are rarely
# used if ever.

sub ActivityHandler						# Database, CGI, Param
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Configuration tags	

	my $data     = $database->{'HANDLE'};			# Database connection handle

	my $puppet;						# Puppet to handle
	my $active;						# Activity flag


	#####################
	# Program area	

	# Find appropraite values
	$active = (defined($param->{$config->{'TxtActive'}})) ? undef : time;
	$puppet = (defined($param->{$config->{'TxtActive'}})) ? $param->{'IPUPPET'}: $param->{'APUPPET'};

	# Update tables
	$data->do("UPDATE Puppet
		SET PuppetActive=?
		WHERE PuppetName=?
		  AND PuppeteerLogin=?", {},
	 $active,
	 $puppet,
	 $param->{'USER'});

	# Cleanup
	$param->Cleanup(
		'PUPPET',
		'APUPPET',
		'IPUPPET',
		$config->{'TxtActive'},
		$config->{'TxtInactive'}
	 );
}


#####################
# Check Data
#  
# Arbitrary sub-routine designed to be used to check validity of data.  Made
# to hide ugliness of checks also centralize adapting of checks.

sub CheckData							# Check value, $choice, Database, User
{
	#####################
	# Data members
	my $value    = shift;					# Value to check 
	my $choice   = shift;					# Choice of the matter
	my $database = shift;					# Possible database handle
	my $user     = shift;					# User


	#####################
	# Program area

	# Password Check
	if ($choice == 0)
	{
		# Retreive Password
		my ($crypt) = $database->DataGet("SELECT PuppeteerPassword
			FROM  Puppeteer
			WHERE PuppeteerLogin=?", $user);	


		# Verify existence of user and validity of password
		if ((defined($crypt)) && ($crypt eq crypt($value, $crypt)))
		{ return 1; }					# Return true
	}

	# Length
	elsif ($choice == 1)
	{
		# Lenght of a minumum of 3 characters
		if (length($value) > 3)
		{ return 1; }					# Return true
	}

	# E-Mail
	elsif ($choice == 2)
	{
		# Check for @ and . lenght
		if ( ($value =~ /\@/) && ($value =~ /\./) && (length($value) > 5) )
		{ return 1; }					# Return true
	}

	return 0; 						# False if all else fails
}

# Alias for Password validity
sub CheckDataPassword
{ CheckData($_[2], 0, $_[0], $_[1]); }

# Alias for Lenght
sub CheckDataLenght
{ CheckData(shift, 1); }

# Alias for E-Eail validity
sub CheckDataEMail
{ CheckData(shift, 1); }


#####################
# Information Handler
#
# Will take the parameters and depending on information received can change the
# passwords and email.  The rest is considered non-essential.

sub InformationHandler						# Database, CGI, Param
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Configuration hash
	
	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $table    = new Ethereal::Table($cgi);		# Table handler
	
	my $choice   = 0;					# Choice of action


	#####################
	# Program area	
	
	# Display of information
	$table->MakeTop();

	# Primary security check
	# Password Validity
	if (CheckDataPassword($database, $param->{'USER'}, $param->{'OLDPASS'}))
	{ 
		# Increment by one
		$choice++; 
	}
	else
	{ 
		# Display error message
		$table->MakeSingle($cgi->h1("<CENTER>$config->{'ErrPassword'}</CENTER>")); 
	}


	# Password change
	if (defined($param->{'ONEPASS'}))
	{
		# Valid new passwords
		if ((CheckDataLenght($param->{'ONEPASS'})) && ($param->{'ONEPASS'} eq $param->{'TWOPASS'}) )
		{
			# Inrement by one 
			$choice++; 
		}
		else
		{
			# Display error message 
			$table->MakeSingle($cgi->h1("<CENTER>$config->{'ErrMismatch'}</CENTER>")); 
		}
	}	

	# E-Mail change
	elsif (defined($param->{'EMAIL'}))
	{
		# Valid new E-Mail
		if (CheckDataEMail($param->{'EMAIL'}))
		{	
			# Inrement by one 
			$choice++; 
		}
		else
		{ 
			# Display error message
			$table->MakeSingle($cgi->h1("<CENTER>$config->{'ErrEMail'}</CENTER>")); 
		}
	}

	# If both tests pass, change data
	if ($choice == 2)
	{
		# Password change
		if ( defined($param->{'ONEPASS'}) )
		{
			# Create random string
			my $rand = md5_hex(rand());
			$rand = substr($rand, 0, 14);			

			# Insert new password
			$data->do("UPDATE Puppeteer 
				SET   PuppeteerPassword=? 
				WHERE PuppeteerLogin=?", {}, 

			 crypt($param->{'ONEPASS'}, $rand), 
		 	 $param->{'USER'});
		}
		
		# EMail change
		elsif ( defined($param->{'EMAIL'}) )
		{
			# Insert new password
			$data->do("UPDATE Puppeteer 
				SET PuppeteerEMail=? 
				WHERE PuppeteerLogin=?", {}, 
			 $param->{'EMAIL'}, 
			 $param->{'USER'});
		}

		# Notify of Success
		$table->MakeSingle($cgi->h1("<CENTER>$config->{'TagConfSuccess'}</CENTER>"));
	}

	# Table bottom
	$table->MakeBottom();
}


#####################
# InitialDisplay
#
# Handles the general output of the initial display screen.  Which will allow one
# to change password, E-Mail addresses and/or passwords.

sub InitialDisplay						# Database, CGI, Param
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Configuration hash

	my $table    = new Ethereal::Table($cgi);
	my $url      = $cgi->url();				# Self referencing URL

	my $iactv;						# Inline puppet active/suspend
	my $imail;						# Inline mail change
	my $ipass;						# Inline password change

	my $active;						# Active handles
	my $inactive;						# Inactive handles

	my @active;						# Active handles
	my @inactive;						# Inactive handles
	
	my %login;						# Login hash
	

	#####################
	# Program area	
	
	# Link with hashes
	$database->GetHashLogin(\%login);

	# Generate Inlines
	($iactv) = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'});
	($imail) = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'CMAIL', 'True');
	($ipass) = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'PASSWORD', 'True');

	
	# Create Inline links
	print "<CENTER><A HREF=\"$url$iactv\">$config->{TagSuspend}</A>\n",
	      "<A HREF=\"$url$imail\">$config->{TagEmail}</A>\n",
	      "<A HREF=\"$url$ipass\">$config->{TagPassword}</A></CENTER>\n<BR><BR>\n";

	# Password change
	if (defined($param->{'PASSWORD'}))
	{
		# Display of information
		# Title
		print "$config->{TagConfPassword}";

		# Form
		print $cgi->start_form();
		print $param->EmbedNormal($cgi, %{$param}, 'PASSWORD', 'True'), "\n";

		# Password section
		$table->MakeTop();
		$table->MakeValid($login{'TagPasswordOld'}, $cgi->password_field('OLDPASS', '', 10, 8));
		$table->MakeValid($login{'TagPassword'}, $cgi->password_field('ONEPASS', '', 10, 8));
		$table->MakeValid($login{'TagPasswordRepeat'}, $cgi->password_field('TWOPASS', '', 10, 8));
		$table->MakeBlank();
	
		# Submit and bottom
		$table->MakeValid($cgi->submit( $login{'TxtSubmitButtonSub'} ), $cgi->reset( $login{'TxtResetButton'} ));
		$table->MakeBottom();

		# Embed
		print $cgi->end_form(), "\n";
		
		# Cleanup
		$param->Cleanup('PASSWORD');
	}


	# Email address change
	elsif (defined($param->{'CMAIL'}))
	{
		# Display of information
		# Title
		print "$config->{TagConfEMail}\n";

		# Form
		print $cgi->start_form();
		print $param->EmbedNormal($cgi, %{$param}, 'CMAIL', 'True');


		# Email section
		$table->MakeTop();
		$table->MakeValid($login{'TagPassword'}, $cgi->password_field('OLDPASS', '', 10, 8));
		$table->MakeValid($login{'TagEMail'}, $cgi->textfield('EMAIL', '', 20, 45));
		$table->MakeBlank();

		# Submit and bottom
		$table->MakeValid($cgi->submit( $login{'TxtSubmitButtonSub'} ), $cgi->reset( $login{'TxtResetButton'} ));
		$table->MakeBottom();

		# Embed
		print $cgi->end_form(), "\n";

		# Cleanup
		$param->Cleanup('EMAIL');
	}

	# Puppet Ignore/Suspend
	else
	{
		# Populate arrays
		$database->GetListPuppetYours(\@active, $param->{'USER'});
		$database->GetListPuppetSuspended(\@inactive, $param->{'USER'});

		# Format lists
		$active   = $cgi->scrolling_list('APUPPET', \@active,   $active[0],   5);
		$inactive = $cgi->scrolling_list('IPUPPET', \@inactive, $inactive[0], 5);

		# Display of information
		# Title
		print "$config->{TagConfActivate}\n";

		# Form
		print $cgi->start_form();
		print $param->EmbedNormal($cgi, %{$param}, 'PUPPET', 'True');

		# Activation section
		$table->MakeTop();
		$table->MakeSingle($config->{'TagActive'});
		$table->MakeValid($active,   $cgi->submit($config->{'TxtInactive'}));
		$table->MakeBlank();
		$table->MakeSingle($config->{'TagInactive'});
		$table->MakeValid($inactive, $cgi->submit($config->{'TxtActive'}));

		# Bottom of page
		$table->MakeBottom();

		# Embed
		print $cgi->end_form(), "\n";
	}
}
